// +FHDR------------------------------------------------------------
//                 Copyright (c) 2022 SmartLogic.
//                       ALL RIGHTS RESERVED
// -----------------------------------------------------------------
// Filename      : ucp_pcie_traffic.h
// Author        : xianfeng.du
// Created On    : 2022-06-25
// Last Modified :
// -----------------------------------------------------------------
// Description:
//
//
// -FHDR------------------------------------------------------------

#ifndef __PCIE_TRAFFIC_H__
#define __PCIE_TRAFFIC_H__

#include "dw_pcie_ep.h"
#include "msg_transfer_queue.h"

#define MAX_CH_NUM_UCP_DMA    MAX_CH_NUM_DW_DMA
#define MAX_THREAD_NUM_UCP_DMA  128 //32 //

#define MAX_DESC_NUM  128

#define MAX_Q_NUM_DL_TRAFFIC UCP4008_TRAFFIC_MAX_NUM
#define MAX_Q_NUM_UL_TRAFFIC UCP4008_TRAFFIC_MAX_NUM


//DMAͨ״̬Լtrafficthread idx
typedef union DmaChStateIdx {
    uint32_t value;
    struct {
        uint16_t state;
        uint16_t thread_Idx;
    };
} DmaChStateIdx_t;


typedef enum eUcpPcieDmaState {
    UCP_DMA_STATE_IDLE,
    UCP_DMA_STATE_WORKING,
    UCP_DMA_STATE_FINISHED
} UcpPcieDmaState_e;

typedef enum ePcieEpStatusType {
	epInit = 1,
	epReady,
	epRunning
} PcieEpStatusType_e;

typedef struct tUcpPcieDmaServiceTraffic {
    uint32_t served_traffic_insid;//С
	uint32_t served_traffic_queid;//queue
    uint32_t thread_idx;	
} UcpPcieDmaServiceTraffic_t;

//DMAͨ״̬ṹ
typedef struct tUcpPcieDmaChannel {
    UcpPcieDmaState_e   state;//DMAͨ״̬лæ
    UcpPcieDmaServiceTraffic_t served_traffic; //DMAͨtraffic
    uint32_t *finished; //ǷѾ
} UcpPcieDmaCh_t;

typedef struct tUcpPcieDmaComplete {
    uint32_t finished;
    uint32_t extInc;//ݰ
    uint32_t intInc;
} UcpPcieDmaComplete_t;

//DMA߳
typedef struct tUcpPcieDmaThread {
    uint32_t size;
    uint32_t mask;//threadһ32ѭʹõģinoutȡģȡģthreadģֵǴ0ʼ31
    uint32_t in; //ʾͨʼDMA
    uint32_t out; //ʾͨDMAѾɣ
    UcpPcieDmaComplete_t thread[MAX_THREAD_NUM_UCP_DMA];
} UcpPcieDmaThread_t;

//ӦͨŵĽṹ
typedef struct tUcpExternalTraffic {
    uint32_t mask;//DMAmaskDMAǻѭʹãÿͨ128
    uint32_t epCntIng;//ͨյݰĸۼӣ128Ҫ127ȡģ
    uint32_t* buf; // point to start of bufferʵշݰ
    uint32_t  offset; // offset to buf to hold next data segment
    uint8_t  lastEop; // EOP of last data segment
    UcpPcieDmaThread_t dma;
} UcpExtTraffic_t;

//PCIE·״̬
typedef struct tPcieLinkStatus {
	uint32_t devEnable;
	uint32_t devReady;
} PcieLinkStatus_t;

//RCEPͨŵcounterṹ
typedef struct tPcieTrafficCounter {
	uint32_t dlDescNum[MAX_INSTANCE_NUM][MAX_Q_NUM_DL_TRAFFIC];//ͨĸ̶Ϊ128
	uint32_t dlTxCounter[MAX_INSTANCE_NUM][MAX_Q_NUM_DL_TRAFFIC];//RC䣬RCдݰĸ
	uint32_t dlRxCounter[MAX_INSTANCE_NUM][MAX_Q_NUM_DL_TRAFFIC];//EPյݰĸ
	//int8_t dlDescIdx[MAX_Q_NUM_DL_TRAFFIC][MAX_DESC_NUM];//ͨDMA128

	uint32_t ulDescNum[MAX_INSTANCE_NUM][MAX_Q_NUM_UL_TRAFFIC];//ͨĸ̶Ϊ12s8
	uint32_t ulTxCounter[MAX_INSTANCE_NUM][MAX_Q_NUM_UL_TRAFFIC];//EP෢ͳȥݰ
	uint32_t ulRxCounter[MAX_INSTANCE_NUM][MAX_Q_NUM_UL_TRAFFIC];//RC䣬RCյݰĸ
	//int8_t ulDescIdx[MAX_Q_NUM_UL_TRAFFIC][MAX_DESC_NUM];//ͨDMA128	
} PcieTrafficCounter_t;

/*
typedef struct tPcieEpCounterMirror {
	volatile uint32_t dlRxCounter[MAX_Q_NUM_DL_TRAFFIC];
	volatile uint32_t ulTxCounter[MAX_Q_NUM_UL_TRAFFIC];
} PcieEpCounterMirror_t __attribute__((__aligned__(NPU_CACHE_LINE_SZ)));
*/
//RCEPͨŵDMARCphyAddrlength
typedef struct tPcieTraffiDescriptor {
    uint64_t phyAddr;  // pointer to start physical address of a buffer in NPU memory
    uint32_t length;    // length of content to be sent in bytes
    uint8_t  eop;       // end of packet, 0=not the last descriptor for a whole message, 1=last descriptor
    uint8_t  type;      // packet type, 0=data, 1=control for data, 2=other control, this field is not used for OAM & FH
    uint16_t rsv;
} PcieTrafficDescriptor_t;

//һԴ9ҵշtaskд
typedef struct tUcpTaskDescriptor{
    PcieTrafficDescriptor_t  dl[MAX_INSTANCE_NUM][MAX_Q_NUM_DL_TRAFFIC][MAX_DESC_NUM];
    PcieTrafficDescriptor_t  ul[MAX_INSTANCE_NUM][MAX_Q_NUM_UL_TRAFFIC][MAX_DESC_NUM];
} UcpTaskDescriptor_t;

//pcie command enum
typedef enum pcieCommandType{
	PCI_ENDPOINT_UCPEP_MAGIC,
	PCI_ENDPOINT_UCPEP_COMMAND,
	PCI_ENDPOINT_UCPEP_EP_ED,
	PCI_ENDPOINT_UCPEP_RC_STATUS,
	PCI_ENDPOINT_UCPEP_EP_STATUS,
	PCI_ENDPOINT_UCPEP_DMA_STATUS,
	PCI_ENDPOINT_UCPEP_UPPER_DST_ADDR,
	PCI_ENDPOINT_UCPEP_SIZE,
	PCI_ENDPOINT_UCPEP_CHECKSUM,
	PCI_ENDPOINT_UCPEP_IRQ_TYPE,
	PCI_ENDPOINT_UCPEP_IRQ_NUMBER,
	PCI_ENDPOINT_UCPEP_TIME_L,
	PCI_ENDPOINT_UCPEP_TIME_R,
	PCI_ENDPOINT_UCPEP_TEST,
}pcieCommand;
#define PCI_UCPEP_EP_HEARTBEAT        0x5A5A5A5A
#define PCI_UCPEP_RC_HEARTBEAT        0xA5A5A5A5
#define PCI_HEARTBEAT_MAXTIME         100


//barռݽṹ壬barռշDMAll
typedef struct tPcieEpMemBarStruct {
	PcieLinkStatus_t     status;//PCIE·״̬      ַ 0x08740000   // 8ֽ
	SyncInfo_t		 	 trafficSyncInfo;	//   0x08740000+8  //
	transfer_type_info_s traffic[MAX_INSTANCE_NUM][TRANSFER_TYPE_NUM];//ҵİҵϢ  0x08740000+ 0x18               //8+16)ֽ 
	PcieTrafficCounter_t cnt; //ÿͨշݰ״̬ṹ //0x08740000+ 0x1F8  //(8+16+480)ֽ
	UcpTaskDescriptor_t  desc;//RCEP֮DMAշݵ//  6*128*2=  1536ֽ//  0x08740000+ 0x288                  (8+16+480+144)= 648
	uint32_t             command[14];//RCEPͨŵcommand, //  0x08740000+ 0x888                  88A=(8+16+480+144+1536)= 2184
	char 				 log_buf[1024];//ڽEPlogRC
} PcieEpMemBarStruct_t;

//PCIEĸṹ
typedef struct tUcpPcieEpMemInfo {
    PcieEpMemBarStruct_t*   	pUcpMemBar;//barռṹ壬RCEP֮Ϣݣ
	
	DwPcieDmaLinkListNode_t*	pDmaDesc;//DMA Link listDMAݰϢԴ
	
    UcpPcieDmaCh_t*         	pDmaDlCh[MAX_CH_NUM_UCP_DMA];//DMAͨ״̬ͨҵid
    UcpPcieDmaCh_t*         	pDmaUlCh[MAX_CH_NUM_UCP_DMA];//DMAͨ״̬ͨҵid
	
    PcieDmaBuffer_t*  			pRdmaChains[MAX_CH_NUM_UCP_DMA];//ͨbufferDMALL
    PcieDmaBuffer_t*  			pWdmaChains[MAX_CH_NUM_UCP_DMA];//дͨbufferͨҵid
	
    UcpExtTraffic_t*        	pExtDlTraffic[MAX_INSTANCE_NUM][UCP4008_TRAFFIC_MAX_NUM];//[MAX_Q_NUM_DL_TRAFFIC]; //ӦҵͨŽṹ
    UcpExtTraffic_t*        	pExtUlTraffic[MAX_INSTANCE_NUM][UCP4008_TRAFFIC_MAX_NUM];//[MAX_Q_NUM_UL_TRAFFIC]; //ӦҵͨŽṹ	
} UcpPcieEpMemInfo_t;

UcpPcieEpMemInfo_t* get_ucp_pcie_mem_info(void);
void ucp_pcie_worker(void);
void ucp_pcie_reset(void);
int ucp_pcie_recfg_reset(void);

void ucp_pcie_reset_bar(void);
void debug_log(uint8_t level, const char *fmt, ...);
void no_debug_log(const char *fmt, ...);

void ucp_testSpeed_pcie_worker(void);

#define PCIE_ERROR_LOG(fmt, args...)  debug_log(LOG_ERROR, "[ERROR]:" fmt "\n", ##args)  //debug_log(fmt, ##args)

//#define PCIE_DEBUG_LOG(fmt, args...)   debug_log(fmt, ##args) 

#define PCIE_DEBUG_LOG(fmt, args...)     debug_log(LOG_DEBUG, "[PCIE_LOG]:" fmt "\n", ##args) //no_debug_log(fmt, ##args)

#endif
